/*****************************************************************************
 * HASM64                                                                    *
 *****************************************************************************/

/*****************************************************************************
 * File name:     symtab.c                                                   *
 * Created:       1997-09-04 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2004-05-31 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 1997-2004 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by  the Free Software Foundation.                                         *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: symtab.c 21 2004-05-31 15:34:10Z hampa $ */


#include "hasm64.h"


symbol_t *sym_new (const char *name, unsigned val)
{
  symbol_t *sym;

  sym = (symbol_t *) malloc (sizeof (symbol_t));
  if (sym == NULL) {
    return (NULL);
  }

  sym->next = NULL;
  sym->name = NULL;
  sym->val = val;
  sym->pass = 0;

  if (name != NULL) {
    sym->name = (char *) malloc (strlen (name) + 1);
    if (sym->name == NULL) {
      free (sym);
      return (NULL);
    }

    strcpy (sym->name, name);
  }

  return (sym);
}

void sym_del (symbol_t *sym)
{
  if (sym != NULL) {
    free (sym->name);
    free (sym);
  }
}

void sym_set_val (symbol_t *sym, unsigned val)
{
  sym->val = val;
}

unsigned sym_get_val (symbol_t *sym)
{
  return (sym->val);
}

void sym_set_name (symbol_t *sym, const char *name)
{
  sym->name = (char *) realloc (sym->name, strlen (name) + 1);
  if (sym->name == NULL) {
    return;
  }

  strcpy (sym->name, name);
}

const char *sym_get_name (symbol_t *sym)
{
  return (sym->name);
}


void st_init (symtab_t *st)
{
  st->cnt = 0;
  st->sym = NULL;
}

symtab_t *st_new (void)
{
  symtab_t *st;

  st = (symtab_t *) malloc (sizeof (symtab_t));
  if (st == NULL) {
    return (NULL);
  }

  st_init (st);

  return (st);
}

void st_free (symtab_t *st)
{
  symbol_t *sym;

  while (st->sym != NULL) {
    sym = st->sym;
    st->sym = st->sym->next;
    sym_del (sym);
  }
}

void st_del (symtab_t *st)
{
  if (st != NULL) {
    st_free (st);
    free (st);
  }
}

void st_sym_add (symtab_t *st, symbol_t *sym)
{
  sym->next = st->sym;
  st->sym = sym;

  st->cnt += 1;
}

void st_sym_rm (symtab_t *st, symbol_t *sym)
{
  symbol_t *sym1, *sym2;

  if (st->sym == NULL) {
    return;
  }

  if (st->sym == sym) {
    st->cnt -= 1;
    st->sym = st->sym->next;
    sym->next = NULL;
    return;
  }

  sym1 = st->sym;
  sym2 = sym1->next;

  while (sym2 != NULL) {
    if (sym2 == sym) {
      st->cnt -= 1;
      sym1->next = sym2->next;
      sym->next = NULL;
      return;
    }

    sym1 = sym2;
    sym2 = sym2->next;
  }
}

symbol_t *st_sym_get (symtab_t *st, const char *name)
{
  symbol_t *sym;

  sym = st->sym;

  while (sym != NULL) {
    if (strcmp (sym->name, name) == 0) {
      return (sym);
    }

    sym = sym->next;
  }

  return (NULL);
}

int st_dump (symtab_t *st, FILE *fp)
{
  int      done;
  unsigned val1, val2;
  symbol_t *sym;

  val2 = 0;

  do {
    done = 1;
    val1 = val2;
    val2 = ~0U;

    sym = st->sym;
    while (sym != NULL) {
      if (sym->val <= val1) {
        if (sym->val == val1) {
          fprintf (fp, "%04X: %s\n", sym->val, sym->name);
        }
      }
      else if (sym->val < val2) {
        val2 = sym->val;
        done = 0;
      }

      sym = sym->next;
    }
  } while (done == 0);

  return (0);
}

int st_dump_file (symtab_t *st, char *fname)
{
  int  r;
  FILE *fp;

  fp = fopen (fname, "wb");
  if (fp == NULL) {
    return (1);
  }

  r = st_dump (st, fp);

  fclose (fp);

  return (r);
}
